/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file adc.c
 *
 */

#include "adc.h"


// ------------------------------------------------------------- PRIVATE MACROS

#define ADC_DUMMY         0
#define ADC_START_BIT     0x4
#define ADC_SINGLE_ENDED  0x2
#define ADC_DATA_RESOL    4095

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS

/**
 * @brief Generic Transfer function.
 *
 * @param obj  SPI module object.
 * @param tx_data  Input buffer to be transferred.
 *
 * @returns 12-bit value of AD conversion.
 *
 * @description This function returns a 12-bit result of AD conversion
 * depending on the sent command.
 */
static uint16_t
adc_spi_master_transfer( spi_master_t *obj, spi_master_data_t *tx_data );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void
adc_cfg_setup( adc_cfg_t *cfg )
{
    cfg->sdo = HAL_PIN_NC;
    cfg->sdi = HAL_PIN_NC;
    cfg->sck = HAL_PIN_NC;
    cfg->cs  = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM;
    cfg->spi_mode  = SPI_MODE_0;

    cfg->vref = ADC_VREF_3300MV;
}

adc_err_t
adc_init( adc_t *ctx, adc_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->sdo;
    spi_cfg.pins.mosi   = cfg->sdi;
    spi_cfg.chip_select = cfg->cs;

    if (spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS)
    {
        return ADC_ERR_INIT_DRV;
    }

    spi_master_set_dummy_data( &ctx->spi, ADC_DUMMY );

    if (cfg->vref == ADC_VREF_3300MV)
    {
        ctx->vref = 3300;
    }
    else if (cfg->vref == ADC_VREF_4096MV)
    {
        ctx->vref = 4096;
    }
    else
    {
        ctx->vref = 5000;
    }

    ctx->ch0 = 0;
    ctx->ch1 = 0;
    ctx->ch2 = 0;
    ctx->ch3 = 0;
    ctx->ch01 = 0;
    ctx->ch10 = 0;
    ctx->ch23 = 0;
    ctx->ch32 = 0;

    return ADC_OK;
}

uint16_t
adc_get_single_ended_ch( adc_t *ctx, adc_ch_t channel )
{
    spi_master_data_t tx_data[ 2 ];
    uint16_t adc_read;
    float res;

    tx_data[ 0 ] = ADC_START_BIT | ADC_SINGLE_ENDED;
    tx_data[ 1 ] = channel << 6;

    adc_read = adc_spi_master_transfer( &ctx->spi, tx_data );

    res = adc_read;
    res /= ADC_DATA_RESOL;
    res *= ctx->vref;

    switch (channel)
    {
        case ADC_CH0_OR_CH01 :
        {
            ctx->ch0 = (uint16_t)res;
        break;
        }
        case ADC_CH1_OR_CH10 :
        {
            ctx->ch1 = (uint16_t)res;
        break;
        }
        case ADC_CH2_OR_CH23 :
        {
            ctx->ch2 = (uint16_t)res;
        break;
        }
        case ADC_CH3_OR_CH32 :
        {
            ctx->ch3 = (uint16_t)res;
        break;
        }
        default :
        {
        break;
        }
    }

    return (uint16_t)res;
}

uint16_t
adc_get_differential_ch( adc_t *ctx, adc_ch_t channel )
{
    spi_master_data_t tx_data[ 2 ];
    uint16_t adc_read;
    float res;

    tx_data[ 0 ] = ADC_START_BIT;
    tx_data[ 1 ] = channel << 6;

    adc_read = adc_spi_master_transfer( &ctx->spi, tx_data );

    res = adc_read;
    res /= ADC_DATA_RESOL;
    res *= ctx->vref;

    switch (channel)
    {
        case ADC_CH0_OR_CH01 :
        {
            ctx->ch01 = (uint16_t)res;
        break;
        }
        case ADC_CH1_OR_CH10 :
        {
            ctx->ch10 = (uint16_t)res;
        break;
        }
        case ADC_CH2_OR_CH23 :
        {
            ctx->ch23 = (uint16_t)res;
        break;
        }
        case ADC_CH3_OR_CH32 :
        {
            ctx->ch32 = (uint16_t)res;
        break;
        }
        default :
        {
        break;
        }
    }

    return (uint16_t)res;
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static uint16_t
adc_spi_master_transfer( spi_master_t *obj, spi_master_data_t *tx_data )
{
    spi_master_transfer_data_t block;
    spi_master_data_t rx_data[ 3 ];
    uint16_t ret_val;

    block.tx_buffer = tx_data;
    block.tx_length = 2;
    block.rx_buffer = rx_data;
    block.rx_length = 3;

    spi_master_start( obj );
    spi_master_transfer( obj, &block );
    spi_master_stop( obj );

    ret_val = rx_data[ 1 ] & 0xF;
    ret_val <<= 8;
    ret_val |= rx_data[ 2 ];

    return ret_val;
}

// ------------------------------------------------------------------------ END
